#include <extdll.h>		// always
#include <sdk_util.h>
#include <meta_api.h>

#include "LogEvents.h"
#include "StandardParser.h"
#include "LogListener.h"

#define CALL_LIST_SIZE ( HIGHEST_EVENT_CODE + 1 )

LogListener* callList[ CALL_LIST_SIZE ]; //used for a simple index table by log event.  Each
                                         //array cell is the head of a linked list
BOOL clean = FALSE;


//recursivly deletes a call list chain
void delList( LogListener* head )
{
    if( head )
    {
        delList( head->next );
        delete head;
    }
}


//called at the end of a map so that the next map things are ready to use (AM plugins are gonig to register each map)
void callListCleanup()
{
    for( int x = LOWEST_EVENT_CODE; x < CALL_LIST_SIZE; x++ )
    {
        if( callList[ x ] )
        {
            delList( callList[ x ] );
            callList[ x ] = 0;
        }
    }
}




void printListeners(void)
{
    LOG_CONSOLE( PLID, "\nAMX LogD callback registration information :" );
    LOG_CONSOLE( PLID, "Event Name          Event Code   Plugin" );
    char* name;

    for( int x = 0; x < CALL_LIST_SIZE; x++ )
    {
        LogListener* current = callList[ x ];
        name = transName( x );
        while( current )
        {
            LOG_CONSOLE( PLID, UTIL_VarArgs( "%-20s%-13i%s", name, x, current->plugin->name) );
        	current = current->next;
        }
    }

}

void initializeListenerModule()
{
    for( int z = 0; z < CALL_LIST_SIZE; z++ )
    {
        callList[ z ] = 0;      //all pointers are now null
    }
    REG_SVR_COMMAND("logd_reginfo", printListeners);
    clean = TRUE;
}

void listenerActivate()
{
    if( !clean )
    {
        LOG_MESSAGE( PLID, "ERROR: callbacks never cleaned from last map, continuing but errors may get issued for multiple registrations." );
    }
    clean = FALSE;
}

void listenerDeactivate()
{
    callListCleanup();
    clean = TRUE;
}
