/* MetaSmall
 *
 * (c) Copyright 2002, OLO
 * No warranties of any kind.
 *
 */

#ifndef __MODULES_H__
#define __MODULES_H__

#include "amx.h"

#undef DLLEXPORT
#ifndef __linux__
  #define DLLEXPORT   __declspec(dllexport)
#else
  #define DLLEXPORT
  #define WINAPI
#endif

#undef C_DLLEXPORT
#define C_DLLEXPORT extern "C" DLLEXPORT

#define MS_IVERS  2

struct module_info_s {
  const char* name;
  const char* author;
  const char* version;
  int ivers;
  long int serial;
};

struct plugin_t {
  AMX amx; // this is the head of script and its code
  void* code;
  
  // rest of values below comes from metasmall
  char name[32];
  char version[32];
  char title[32];
  char author[32];
  int id;
  int paused_fun;
  int paused;
  plugin_t *next, *prev;

  // for mods
  int iuser1, iuser2, iuser3, iuser4, iuser5;
  float fuser1, fuser2, fuser3,fuser4, fuser5;
  cell *cuser1, *cuser2, *cuser3;
  char *suser1, *suser2, *suser3;
  void* data;
};

struct funcall_t {
  plugin_t* plugin;
  int func;
  int id;
  funcall_t *next, *prev;

  // for mods
  int iuser1, iuser2, iuser3, iuser4, iuser5;
  float fuser1, fuser2, fuser3,fuser4, fuser5;
  cell *cuser1, *cuser2, *cuser3;
  char *suser1, *suser2, *suser3;
  void* data;
};

// Stuff which comes with Small scripting language
struct pfnamx_engine_g {
  uint16_t*   (*pfnamx_Align16)(uint16_t *);                    // value
  uint32_t*   (*pfnamx_Align32)(uint32_t *);                    // value
  int     (*pfnamx_Allot)(AMX*, int, cell*, cell**);              // amx, length, amx_addr, phys_addr
  int     (*pfnamx_Callback)(AMX*, cell , cell*, cell*);            // amx, index,result,params
  int     (*pfnamx_Clone)(AMX*, AMX*, void*);                 // amxClone, amxSrc, data
  int     (*pfnamx_Debug)(AMX*); // default debug procedure, does nothing   // amx
  int     (*pfnamx_Exec)(AMX*, cell*, int , int , ...);           // amx, return val, index, num_params, ...
  int     (*pfnamx_Execv)(AMX*, cell*, int , int, cell[]);          // amx, return val, index, num_params, param[]
  int     (*pfnamx_FindPublic)(AMX*, char*, int*);              // amx, func name, index
  int     (*pfnamx_FindPubVar)(AMX*, char*, cell*);             // anx, var name, amx_addr
  int     (*pfnamx_FindTagId)(AMX*, cell , char*);              // amx. tag_id, tagname
  int     (*pfnamx_Flags)(AMX*,uint16_t *);                 // amx, flags
  int     (*pfnamx_GetAddr)(AMX*,cell ,cell**);               // amx, amx_addr, phys_addr
  int     (*pfnamx_GetPublic)(AMX*, int , char*);               // amx, index, funcname
  int     (*pfnamx_GetPubVar)(AMX*, int , char*, cell*);            // amx, index, varname, amx_addr
  int     (*pfnamx_GetString)(char*dest,cell*);               // dest, source
  int     (*pfnamx_GetTag)(AMX*, int , char*, cell*);             // amx, index, tagname, tag_id
  int     (*pfnamx_GetUserData)(AMX*, long , void **);            // amx, tag, ptr
  int     (*pfnamx_Init)(AMX*, void *);                   // amx, program
  int     (*pfnamx_InitJIT)(AMX*, void *, void *);              // amx, reloc_table, native_code
  int     (*pfnamx_MemInfo)(AMX*, long*, long*, long*);           // amx, codesize, datasize, stackheap
  int     (*pfnamx_NameLength)(AMX*, int*);                 // amx, length
  AMX_NATIVE_INFO * (*pfnamx_NativeInfo)(char*,AMX_NATIVE );            // name, func
  int     (*pfnamx_NumPublics)(AMX*, int*);                 // amx, number
  int     (*pfnamx_NumPubVars)(AMX*, int*);                 // amx, number
  int     (*pfnamx_NumTags)(AMX*, int*);                    // amx, number
  int     (*pfnamx_RaiseError)(AMX*, int );                 // amx, error
  int     (*pfnamx_Register)(AMX*, AMX_NATIVE_INFO*, int );         // amx, nativelist, number
  int     (*pfnamx_Release)(AMX*, cell );                   // amx, amx_addr
  int     (*pfnamx_SetCallback)(AMX*, AMX_CALLBACK );             // amx, callback
  int     (*pfnamx_SetDebugHook)(AMX*, AMX_DEBUG );             // amx, debug
  int     (*pfnamx_SetString)(cell*, char*, int );              // dest, source, pack
  int     (*pfnamx_SetUserData)(AMX*, long , void*);              // amx, tag, prt
  int     (*pfnamx_StrLen)(cell*, int*);                    // amx, cstring, length
};
extern pfnamx_engine_g* g_engAmxFunc;

#define AMX_ALIGN16     (*g_engAmxFunc->pfnamx_Align16)
#define AMX_ALIGN32     (*g_engAmxFunc->pfnamx_Align32)
#define AMX_ALLOT   (*g_engAmxFunc->pfnamx_Allot)
#define AMX_CALLBACK    (*g_engAmxFunc->pfnamx_Callback)
#define AMX_CLONE   (*g_engAmxFunc->pfnamx_Clone)
#define AMX_DEBUG   (*g_engAmxFunc->pfnamx_Debug)
#define AMX_EXEC    (*g_engAmxFunc->pfnamx_Exec)
#define AMX_EXECV   (*g_engAmxFunc->pfnamx_Execv)
#define AMX_FINDPUBLIC  (*g_engAmxFunc->pfnamx_FindPublic)
#define AMX_FINDPUBVAR  (*g_engAmxFunc->pfnamx_FindPubVar)
#define AMX_FINDTAGID   (*g_engAmxFunc->pfnamx_FindTagId)
#define AMX_FLAGS   (*g_engAmxFunc->pfnamx_Flags)
#define AMX_GETADDR     (*g_engAmxFunc->pfnamx_GetAddr)
#define AMX_GETPUBLIC   (*g_engAmxFunc->pfnamx_GetPublic)
#define AMX_GETPUBVAR   (*g_engAmxFunc->pfnamx_GetPubVar)
#define AMX_GETSTRING   (*g_engAmxFunc->pfnamx_GetString)
#define AMX_GETTAG      (*g_engAmxFunc->pfnamx_GetTag)
#define AMX_GETUSERDATA (*g_engAmxFunc->pfnamx_GetUserData)
#define AMX_INIT    (*g_engAmxFunc->pfnamx_Init)
#define AMX_INITJIT     (*g_engAmxFunc->pfnamx_InitJIT)
#define AMX_MEMINFO     (*g_engAmxFunc->pfnamx_MemInfo)
#define AMX_NAMELENGTH  (*g_engAmxFunc->pfnamx_NameLength)
#define AMX_NATIVEINFO  (*g_engAmxFunc->pfnamx_NativeInfo)
#define AMX_NUMPUBLICS  (*g_engAmxFunc->pfnamx_NumPublics)
#define AMX_NUMPUBVARS  (*g_engAmxFunc->pfnamx_NumPubVars)
#define AMX_NUMTAGS     (*g_engAmxFunc->pfnamx_NumTags)
#define AMX_RAISEERROR  (*g_engAmxFunc->pfnamx_RaiseError)
#define AMX_REGISTER    (*g_engAmxFunc->pfnamx_Register)
#define AMX_RELEASE     (*g_engAmxFunc->pfnamx_Release)
#define AMX_SETCALLBACK (*g_engAmxFunc->pfnamx_SetCallback)
#define AMX_SETDEBUGHOOK (*g_engAmxFunc->pfnamx_SetDebugHook)
#define AMX_SETSTRING   (*g_engAmxFunc->pfnamx_SetString)
#define AMX_SETUSERDATA (*g_engAmxFunc->pfnamx_SetUserData)
#define AMX_STRLEN      (*g_engAmxFunc->pfnamx_StrLen)

// Stuff which should make some things more easier
struct pfnms_engine_g {
  int     (*pfnadd_natives)(AMX_NATIVE_INFO*);          // list
  int     (*pfnalloc_memory)(void**,size_t);          // ptr, size
  void    (*pfncall_function)(funcall_t*,int=0,cell=0);     // call funcs list, num param, param
  void    (*pfncopy_amxmemory)(cell*,cell*,int);          // dest, src, len
  int     (*pfncreate_function_call)(funcall_t**, funcall_t**, plugin_t*, const char*);  // funcs, plugin, func name  
  int     (*pfncreate_plugin)(plugin_t**, plugin_t**, const char*, const char*, char[64]); // plugins list, plugin ptr, path, filename, error  
  void    (*pfndelete_function_call)(funcall_t**,funcall_t*);    // call funcs list, func call
  void    (*pfndelete_function_calls)(funcall_t**);       // call funcs list  
  void    (*pfndelete_plugin)(plugin_t**,plugin_t*);           // plugins list, plugin  
  void    (*pfndelete_plugins)(plugin_t**);           // plugins list  
  plugin_t* (*pfnfind_plugin_amx)(plugin_t*,AMX*);          // plugins list, amx  
  plugin_t* (*pfnfind_plugin_index)(plugin_t*,int);         // plugins list, index  
  plugin_t* (*pfnfind_plugin_name)(plugin_t*,const char*);      // plugins list, name  
  char*   (*pfnformat_amxstring)(AMX*, cell*, int ,int& );    // amx, formating, start pos, len
  int   (*pfnfree_memory)(void**);          // ptr
  cell*   (*pfnget_amxaddr)(AMX*,cell );              // amx, cell
  char*   (*pfnget_amxstring)(AMX*,cell,int, int&);       // amx, src, buffor (0-3); len
  const char* (*pfnget_pluginstatus)(plugin_t*);
  int     (*pfnload_amxscript)(AMX*, void**, const char*, char[64]);  // amx, code, path, error_info 
  int     (*pfnpause_function)(plugin_t*, const char*);     // plugin, func name 
  int     (*pfnpause_plugin)(plugin_t*);              // plugin    
  int   (*pfnset_amxscriptnatives)(AMX*,char[64]); 
  int     (*pfnset_amxstring)(AMX*,cell ,const char*,int);    // amx, dest, string, maxlen  
  void    (*pfnset_plugin_info)(plugin_t*, const char*, const char*, const char*); // plugin, title, ver, author  
  int     (*pfnstring_length)(cell*);               // src   
  int     (*pfnunload_amxscript)(void**);              // code  
  int     (*pfnunpause_function)(plugin_t* plugin, const char*);  // plugin, func name 
  int     (*pfnunpause_plugin)(plugin_t*);            // plugin
};

extern pfnms_engine_g* g_engMetaSmallFunc;

#define ADD_NATIVES         (*g_engMetaSmallFunc->pfnadd_natives)
#define ALLOC_MEMORY        (*g_engMetaSmallFunc->pfnalloc_memory)
#define CALL_FUNCTION       (*g_engMetaSmallFunc->pfncall_function)
#define COPY_AMXMEMORY      (*g_engMetaSmallFunc->pfncopy_amxmemory)
#define CREATE_FUNC_CALL    (*g_engMetaSmallFunc->pfncreate_function_call)
#define CREATE_PLUGIN       (*g_engMetaSmallFunc->pfncreate_plugin)
#define DELETE_FUNC_CALL  (*g_engMetaSmallFunc->pfndelete_function_call)
#define DELETE_FUNC_CALLS   (*g_engMetaSmallFunc->pfndelete_function_calls)
#define DELETE_PLUGIN   (*g_engMetaSmallFunc->pfndelete_plugin)
#define DELETE_PLUGINS      (*g_engMetaSmallFunc->pfndelete_plugins)
#define FIND_PLUGIN_BY_AMX    (*g_engMetaSmallFunc->pfnfind_plugin_amx)
#define FIND_PLUGIN_BY_INDEX  (*g_engMetaSmallFunc->pfnfind_plugin_index)
#define FIND_PLUGIN_BY_NAME   (*g_engMetaSmallFunc->pfnfind_plugin_name)
#define FORMAT_AMXSTRING    (*g_engMetaSmallFunc->pfnformat_amxstring)
#define FREE_MEMORY     (*g_engMetaSmallFunc->pfnfree_memory)
#define GET_AMXADDR         (*g_engMetaSmallFunc->pfnget_amxaddr)
#define GET_AMXSTRING       (*g_engMetaSmallFunc->pfnget_amxstring)
#define GET_PLUGINSTATUS    (*g_engMetaSmallFunc->pfnget_pluginstatus)
#define LOAD_AMXSCRIPT      (*g_engMetaSmallFunc->pfnload_amxscript)
#define PAUSE_FUNCTION      (*g_engMetaSmallFunc->pfnpause_function)
#define PAUSE_PLUGIN        (*g_engMetaSmallFunc->pfnpause_plugin)
#define SET_AMXSCRIPTNATIVES  (*g_engMetaSmallFunc->pfnset_amxscriptnatives)
#define SET_AMXSTRING       (*g_engMetaSmallFunc->pfnset_amxstring)
#define SET_PLUGIN_INFO     (*g_engMetaSmallFunc->pfnset_plugin_info)
#define STRING_LENGTH       (*g_engMetaSmallFunc->pfnstring_length)
#define UNLOAD_AMXSCRIPT    (*g_engMetaSmallFunc->pfnunload_amxscript)
#define UNPAUSE_FUNCTION    (*g_engMetaSmallFunc->pfnunpause_function)
#define UNPAUSE_PLUGIN      (*g_engMetaSmallFunc->pfnunpause_plugin)

#endif // __MODULES_H__
